@@ -226,6 +226,10 @@ module ActiveRecord
         @association.last(*args)
       end
 
+      def take(n = nil)
+        @association.take(n)
+      end
+
       # Returns a new object of the collection type that has been instantiated
       # with +attributes+ and linked to this object, but have not yet been saved.
       # You can pass an array of attributes hashes, this will return an array
