@@ -166,11 +166,11 @@ class PostgresqlDataTypeTest < ActiveRecord::TestCase
   end
 
   def test_number_values
-    assert_equal 123.456, @first_number.single
-    assert_equal 123456.789, @first_number.double
-    assert_equal -::Float::INFINITY, @second_number.single
-    assert_equal ::Float::INFINITY, @second_number.double
-    assert_same ::Float::NAN, @third_number.double
+    assert_equal(123.456, @first_number.single)
+    assert_equal(123456.789, @first_number.double)
+    assert_equal(-::Float::INFINITY, @second_number.single)
+    assert_equal(::Float::INFINITY, @second_number.double)
+    assert_same(::Float::NAN, @third_number.double)
   end
 
   def test_time_values
