@@ -1,8 +1,59 @@
 require 'logger'
 require 'abstract_unit'
 require 'active_support/cache'
+require 'dependencies_test_helpers'
+
+module ActiveSupport
+  module Cache
+    module Strategy
+      module LocalCache
+        class MiddlewareTest < ActiveSupport::TestCase
+          def test_local_cache_cleared_on_close
+            key = "super awesome key"
+            assert_nil LocalCacheRegistry.cache_for key
+            middleware = Middleware.new('<3', key).new(->(env) {
+              assert LocalCacheRegistry.cache_for(key), 'should have a cache'
+              [200, {}, []]
+            })
+            _, _, body = middleware.call({})
+            assert LocalCacheRegistry.cache_for(key), 'should still have a cache'
+            body.each { }
+            assert LocalCacheRegistry.cache_for(key), 'should still have a cache'
+            body.close
+            assert_nil LocalCacheRegistry.cache_for(key)
+          end
+
+          def test_local_cache_cleared_on_exception
+            key = "super awesome key"
+            assert_nil LocalCacheRegistry.cache_for key
+            middleware = Middleware.new('<3', key).new(->(env) {
+              assert LocalCacheRegistry.cache_for(key), 'should have a cache'
+              raise
+            })
+            assert_raises(RuntimeError) { middleware.call({}) }
+            assert_nil LocalCacheRegistry.cache_for(key)
+          end
+        end
+      end
+    end
+  end
+end
 
 class CacheKeyTest < ActiveSupport::TestCase
+  def test_entry_legacy_optional_ivars
+    legacy = Class.new(ActiveSupport::Cache::Entry) do
+      def initialize(value, options = {})
+        @value = value
+        @expires_in = nil
+        @created_at = nil
+        super
+      end
+    end
+
+    entry = legacy.new 'foo'
+    assert_equal 'foo', entry.value
+  end
+
   def test_expand_cache_key
     assert_equal '1/2/true', ActiveSupport::Cache.expand_cache_key([1, '2', true])
     assert_equal 'name/1/2/true', ActiveSupport::Cache.expand_cache_key([1, '2', true], :name)
@@ -69,6 +120,10 @@ class CacheKeyTest < ActiveSupport::TestCase
   def test_expand_cache_key_of_true
     assert_equal 'true', ActiveSupport::Cache.expand_cache_key(true)
   end
+
+  def test_expand_cache_key_of_array_like_object
+    assert_equal 'foo/bar/baz', ActiveSupport::Cache.expand_cache_key(%w{foo bar baz}.to_enum)
+  end
 end
 
 class CacheStoreSettingTest < ActiveSupport::TestCase
@@ -79,34 +134,34 @@ class CacheStoreSettingTest < ActiveSupport::TestCase
   end
 
   def test_mem_cache_fragment_cache_store
-    MemCache.expects(:new).with(%w[localhost], {})
+    Dalli::Client.expects(:new).with(%w[localhost], {})
     store = ActiveSupport::Cache.lookup_store :mem_cache_store, "localhost"
     assert_kind_of(ActiveSupport::Cache::MemCacheStore, store)
   end
 
   def test_mem_cache_fragment_cache_store_with_given_mem_cache
-    mem_cache = MemCache.new
-    MemCache.expects(:new).never
+    mem_cache = Dalli::Client.new
+    Dalli::Client.expects(:new).never
     store = ActiveSupport::Cache.lookup_store :mem_cache_store, mem_cache
     assert_kind_of(ActiveSupport::Cache::MemCacheStore, store)
   end
 
-  def test_mem_cache_fragment_cache_store_with_given_mem_cache_like_object
-    MemCache.expects(:new).never
+  def test_mem_cache_fragment_cache_store_with_not_dalli_client
+    Dalli::Client.expects(:new).never
     memcache = Object.new
-    def memcache.get() true end
-    store = ActiveSupport::Cache.lookup_store :mem_cache_store, memcache
-    assert_kind_of(ActiveSupport::Cache::MemCacheStore, store)
+    assert_raises(ArgumentError) do
+      ActiveSupport::Cache.lookup_store :mem_cache_store, memcache
+    end
   end
 
   def test_mem_cache_fragment_cache_store_with_multiple_servers
-    MemCache.expects(:new).with(%w[localhost 192.168.1.1], {})
+    Dalli::Client.expects(:new).with(%w[localhost 192.168.1.1], {})
     store = ActiveSupport::Cache.lookup_store :mem_cache_store, "localhost", '192.168.1.1'
     assert_kind_of(ActiveSupport::Cache::MemCacheStore, store)
   end
 
   def test_mem_cache_fragment_cache_store_with_options
-    MemCache.expects(:new).with(%w[localhost 192.168.1.1], { :timeout => 10 })
+    Dalli::Client.expects(:new).with(%w[localhost 192.168.1.1], { :timeout => 10 })
     store = ActiveSupport::Cache.lookup_store :mem_cache_store, "localhost", '192.168.1.1', :namespace => 'foo', :timeout => 10
     assert_kind_of(ActiveSupport::Cache::MemCacheStore, store)
     assert_equal 'foo', store.options[:namespace]
@@ -179,6 +234,16 @@ module CacheStoreBehavior
     assert_equal 'baz', @cache.fetch('foo') { 'baz' }
   end
 
+  def test_fetch_with_cache_miss_passes_key_to_block
+    cache_miss = false
+    assert_equal 3, @cache.fetch('foo') { |key| cache_miss = true; key.length }
+    assert cache_miss
+
+    cache_miss = false
+    assert_equal 3, @cache.fetch('foo') { |key| cache_miss = true; key.length }
+    assert !cache_miss
+  end
+
   def test_fetch_with_forced_cache_miss
     @cache.write('foo', 'bar')
     @cache.expects(:read).never
@@ -212,21 +277,6 @@ module CacheStoreBehavior
     assert_equal false, @cache.read('foo')
   end
 
-  def test_should_read_cached_numeric_from_previous_rails_versions
-    @old_cache = ActiveSupport::Cache::Entry.create( 1, Time.now )
-    assert_equal( 1, @old_cache.value )
-  end
-
-  def test_should_read_cached_hash_from_previous_rails_versions
-    @old_cache = ActiveSupport::Cache::Entry.create( {}, Time.now )
-    assert_equal( {}, @old_cache.value )
-  end
-
-  def test_should_read_cached_string_from_previous_rails_versions
-    @old_cache = ActiveSupport::Cache::Entry.create( 'string', Time.now )
-    assert_equal( 'string', @old_cache.value )
-  end
-
   def test_read_multi
     @cache.write('foo', 'bar')
     @cache.write('fu', 'baz')
@@ -235,25 +285,42 @@ module CacheStoreBehavior
   end
 
   def test_read_multi_with_expires
-    @cache.write('foo', 'bar', :expires_in => 0.001)
+    time = Time.now
+    @cache.write('foo', 'bar', :expires_in => 10)
     @cache.write('fu', 'baz')
     @cache.write('fud', 'biz')
-    sleep(0.002)
+    Time.stubs(:now).returns(time + 11)
     assert_equal({"fu" => "baz"}, @cache.read_multi('foo', 'fu'))
   end
 
+  def test_fetch_multi
+    @cache.write('foo', 'bar')
+    @cache.write('fud', 'biz')
+
+    values = @cache.fetch_multi('foo', 'fu', 'fud') {|value| value * 2 }
+
+    assert_equal(["bar", "fufu", "biz"], values)
+    assert_equal("fufu", @cache.read('fu'))
+  end
+
+  def test_multi_with_objects
+    foo = stub(:title => "FOO!", :cache_key => "foo")
+    bar = stub(:cache_key => "bar")
+
+    @cache.write('bar', "BAM!")
+
+    values = @cache.fetch_multi(foo, bar) {|object| object.title }
+    assert_equal(["FOO!", "BAM!"], values)
+  end
+
   def test_read_and_write_compressed_small_data
     @cache.write('foo', 'bar', :compress => true)
-    raw_value = @cache.send(:read_entry, 'foo', {}).raw_value
     assert_equal 'bar', @cache.read('foo')
-    assert_equal 'bar', Marshal.load(raw_value)
   end
 
   def test_read_and_write_compressed_large_data
     @cache.write('foo', 'bar', :compress => true, :compress_threshold => 2)
-    raw_value = @cache.send(:read_entry, 'foo', {}).raw_value
     assert_equal 'bar', @cache.read('foo')
-    assert_equal 'bar', Marshal.load(Zlib::Inflate.inflate(raw_value))
   end
 
   def test_read_and_write_compressed_nil
@@ -296,8 +363,8 @@ module CacheStoreBehavior
 
   def test_exist
     @cache.write('foo', 'bar')
-    assert @cache.exist?('foo')
-    assert !@cache.exist?('bar')
+    assert_equal true, @cache.exist?('foo')
+    assert_equal false, @cache.exist?('bar')
   end
 
   def test_nil_exist
@@ -312,14 +379,6 @@ module CacheStoreBehavior
     assert !@cache.exist?('foo')
   end
 
-  def test_read_should_return_a_different_object_id_each_time_it_is_called
-    @cache.write('foo', 'bar')
-    assert_not_equal @cache.read('foo').object_id, @cache.read('foo').object_id
-    value = @cache.read('foo')
-    value << 'bingo'
-    assert_not_equal value, @cache.read('foo')
-  end
-
   def test_original_store_objects_should_not_be_immutable
     bar = 'bar'
     @cache.write('foo', bar)
@@ -374,7 +433,7 @@ module CacheStoreBehavior
     rescue ArgumentError
     end
     assert_equal "bar", @cache.read('foo')
-    Time.stubs(:now).returns(time + 71)
+    Time.stubs(:now).returns(time + 91)
     assert_nil @cache.read('foo')
   end
 
@@ -402,25 +461,12 @@ module CacheStoreBehavior
 end
 
 # https://rails.lighthouseapp.com/projects/8994/tickets/6225-memcachestore-cant-deal-with-umlauts-and-special-characters
-# The error is caused by charcter encodings that can't be compared with ASCII-8BIT regular expressions and by special
+# The error is caused by character encodings that can't be compared with ASCII-8BIT regular expressions and by special
 # characters like the umlaut in UTF-8.
 module EncodedKeyCacheBehavior
-  if defined?(Encoding)
-    Encoding.list.each do |encoding|
-      define_method "test_#{encoding.name.underscore}_encoded_values" do
-        key = "foo".force_encoding(encoding)
-        assert @cache.write(key, "1", :raw => true)
-        assert_equal "1", @cache.read(key)
-        assert_equal "1", @cache.fetch(key)
-        assert @cache.delete(key)
-        assert_equal "2", @cache.fetch(key, :raw => true) { "2" }
-        assert_equal 3, @cache.increment(key)
-        assert_equal 2, @cache.decrement(key)
-      end
-    end
-
-    def test_common_utf8_values
-      key = "\xC3\xBCmlaut".force_encoding(Encoding::UTF_8)
+  Encoding.list.each do |encoding|
+    define_method "test_#{encoding.name.underscore}_encoded_values" do
+      key = "foo".force_encoding(encoding)
       assert @cache.write(key, "1", :raw => true)
       assert_equal "1", @cache.read(key)
       assert_equal "1", @cache.fetch(key)
@@ -429,12 +475,23 @@ module EncodedKeyCacheBehavior
       assert_equal 3, @cache.increment(key)
       assert_equal 2, @cache.decrement(key)
     end
+  end
 
-    def test_retains_encoding
-      key = "\xC3\xBCmlaut".force_encoding(Encoding::UTF_8)
-      assert @cache.write(key, "1", :raw => true)
-      assert_equal Encoding::UTF_8, key.encoding
-    end
+  def test_common_utf8_values
+    key = "\xC3\xBCmlaut".force_encoding(Encoding::UTF_8)
+    assert @cache.write(key, "1", :raw => true)
+    assert_equal "1", @cache.read(key)
+    assert_equal "1", @cache.fetch(key)
+    assert @cache.delete(key)
+    assert_equal "2", @cache.fetch(key, :raw => true) { "2" }
+    assert_equal 3, @cache.increment(key)
+    assert_equal 2, @cache.decrement(key)
+  end
+
+  def test_retains_encoding
+    key = "\xC3\xBCmlaut".force_encoding(Encoding::UTF_8)
+    assert @cache.write(key, "1", :raw => true)
+    assert_equal Encoding::UTF_8, key.encoding
   end
 end
 
@@ -460,6 +517,7 @@ module CacheIncrementDecrementBehavior
     assert_equal 2, @cache.read('foo').to_i
     assert_equal 3, @cache.increment('foo')
     assert_equal 3, @cache.read('foo').to_i
+    assert_nil @cache.increment('bar')
   end
 
   def test_decrement
@@ -469,6 +527,7 @@ module CacheIncrementDecrementBehavior
     assert_equal 2, @cache.read('foo').to_i
     assert_equal 1, @cache.decrement('foo')
     assert_equal 1, @cache.read('foo').to_i
+    assert_nil @cache.decrement('bar')
   end
 end
 
@@ -554,18 +613,61 @@ module LocalCacheBehavior
       result = @cache.write('foo', 'bar')
       assert_equal 'bar', @cache.read('foo') # make sure 'foo' was written
       assert result
+      [200, {}, []]
     }
     app = @cache.middleware.new(app)
     app.call({})
   end
 end
 
+module AutoloadingCacheBehavior
+  include DependenciesTestHelpers
+  def test_simple_autoloading
+    with_autoloading_fixtures do
+      @cache.write('foo', E.new)
+    end
+
+    remove_constants(:E)
+    ActiveSupport::Dependencies.clear
+
+    with_autoloading_fixtures do
+      assert_kind_of E, @cache.read('foo')
+    end
+
+    remove_constants(:E)
+    ActiveSupport::Dependencies.clear
+  end
+
+  def test_two_classes_autoloading
+    with_autoloading_fixtures do
+      @cache.write('foo', [E.new, ClassFolder.new])
+    end
+
+    remove_constants(:E, :ClassFolder)
+    ActiveSupport::Dependencies.clear
+
+    with_autoloading_fixtures do
+      loaded = @cache.read('foo')
+      assert_kind_of Array, loaded
+      assert_equal 2, loaded.size
+      assert_kind_of E, loaded[0]
+      assert_kind_of ClassFolder, loaded[1]
+    end
+
+    remove_constants(:E, :ClassFolder)
+    ActiveSupport::Dependencies.clear
+  end
+end
+
 class FileStoreTest < ActiveSupport::TestCase
   def setup
     Dir.mkdir(cache_dir) unless File.exist?(cache_dir)
     @cache = ActiveSupport::Cache.lookup_store(:file_store, cache_dir, :expires_in => 60)
     @peek = ActiveSupport::Cache.lookup_store(:file_store, cache_dir, :expires_in => 60)
     @cache_with_pathname = ActiveSupport::Cache.lookup_store(:file_store, Pathname.new(cache_dir), :expires_in => 60)
+
+    @buffer = StringIO.new
+    @cache.logger = ActiveSupport::Logger.new(@buffer)
   end
 
   def teardown
@@ -580,6 +682,14 @@ class FileStoreTest < ActiveSupport::TestCase
   include LocalCacheBehavior
   include CacheDeleteMatchedBehavior
   include CacheIncrementDecrementBehavior
+  include AutoloadingCacheBehavior
+
+  def test_clear
+    filepath = File.join(cache_dir, ".gitkeep")
+    FileUtils.touch(filepath)
+    @cache.clear
+    assert File.exist?(filepath)
+  end
 
   def test_key_transformation
     key = @cache.send(:key_file_path, "views/index?id=1")
@@ -592,6 +702,16 @@ class FileStoreTest < ActiveSupport::TestCase
     assert_equal "views/index?id=1", @cache_with_pathname.send(:file_path_key, key)
   end
 
+  # Test that generated cache keys are short enough to have Tempfile stuff added to them and
+  # remain valid
+  def test_filename_max_size
+    key = "#{'A' * ActiveSupport::Cache::FileStore::FILENAME_MAX_SIZE}"
+    path = @cache.send(:key_file_path, key)
+    Dir::Tmpname.create(path) do |tmpname, n, opts|
+      assert File.basename(tmpname+'.lock').length <= 255, "Temp filename too long: #{File.basename(tmpname+'.lock').length}"
+    end
+  end
+
   # Because file systems have a maximum filename size, filenames > max size should be split in to directories
   # If filename is 'AAAAB', where max size is 4, the returned path should be AAAA/B
   def test_key_transformation_max_filename_size
@@ -609,23 +729,48 @@ class FileStoreTest < ActiveSupport::TestCase
     end
   end
 
+  def test_delete_does_not_delete_empty_parent_dir
+    sub_cache_dir = File.join(cache_dir, 'subdir/')
+    sub_cache_store = ActiveSupport::Cache::FileStore.new(sub_cache_dir)
+    assert_nothing_raised(Exception) do
+      assert sub_cache_store.write('foo', 'bar')
+      assert sub_cache_store.delete('foo')
+    end
+    assert File.exist?(cache_dir), "Parent of top level cache dir was deleted!"
+    assert File.exist?(sub_cache_dir), "Top level cache dir was deleted!"
+    assert Dir.entries(sub_cache_dir).reject {|f| ActiveSupport::Cache::FileStore::EXCLUDED_DIRS.include?(f)}.empty?
+  end
+
+  def test_log_exception_when_cache_read_fails
+    File.expects(:exist?).raises(StandardError, "failed")
+    @cache.send(:read_entry, "winston", {})
+    assert @buffer.string.present?
+  end
+
   def test_cleanup_removes_all_expired_entries
     time = Time.now
-    @cache.write('foo', 'bar', :expires_in => 10)
+    @cache.write('foo', 'bar', expires_in: 10)
     @cache.write('baz', 'qux')
-    @cache.write('quux', 'corge', :expires_in => 20)
+    @cache.write('quux', 'corge', expires_in: 20)
     Time.stubs(:now).returns(time + 15)
     @cache.cleanup
-    assert !@cache.exist?('foo')
+    assert_not @cache.exist?('foo')
     assert @cache.exist?('baz')
     assert @cache.exist?('quux')
   end
+
+  def test_write_with_unless_exist
+    assert_equal true, @cache.write(1, "aaaaaaaaaa")
+    assert_equal false, @cache.write(1, "aaaaaaaaaa", unless_exist: true)
+    @cache.write(1, nil)
+    assert_equal false, @cache.write(1, "aaaaaaaaaa", unless_exist: true)
+  end
 end
 
 class MemoryStoreTest < ActiveSupport::TestCase
   def setup
-    @record_size = Marshal.dump("aaaaaaaaaa").bytesize
-    @cache = ActiveSupport::Cache.lookup_store(:memory_store, :expires_in => 60, :size => @record_size * 10)
+    @record_size = ActiveSupport::Cache.lookup_store(:memory_store).send(:cached_size, 1, ActiveSupport::Cache::Entry.new("aaaaaaaaaa"))
+    @cache = ActiveSupport::Cache.lookup_store(:memory_store, :expires_in => 60, :size => @record_size * 10 + 1)
   end
 
   include CacheStoreBehavior
@@ -643,9 +788,9 @@ class MemoryStoreTest < ActiveSupport::TestCase
     @cache.prune(@record_size * 3)
     assert @cache.exist?(5)
     assert @cache.exist?(4)
-    assert !@cache.exist?(3)
+    assert !@cache.exist?(3), "no entry"
     assert @cache.exist?(2)
-    assert !@cache.exist?(1)
+    assert !@cache.exist?(1), "no entry"
   end
 
   def test_prune_size_on_write
@@ -667,12 +812,36 @@ class MemoryStoreTest < ActiveSupport::TestCase
     assert @cache.exist?(9)
     assert @cache.exist?(8)
     assert @cache.exist?(7)
-    assert !@cache.exist?(6)
-    assert !@cache.exist?(5)
+    assert !@cache.exist?(6), "no entry"
+    assert !@cache.exist?(5), "no entry"
     assert @cache.exist?(4)
-    assert !@cache.exist?(3)
+    assert !@cache.exist?(3), "no entry"
     assert @cache.exist?(2)
-    assert !@cache.exist?(1)
+    assert !@cache.exist?(1), "no entry"
+  end
+
+  def test_prune_size_on_write_based_on_key_length
+    @cache.write(1, "aaaaaaaaaa") && sleep(0.001)
+    @cache.write(2, "bbbbbbbbbb") && sleep(0.001)
+    @cache.write(3, "cccccccccc") && sleep(0.001)
+    @cache.write(4, "dddddddddd") && sleep(0.001)
+    @cache.write(5, "eeeeeeeeee") && sleep(0.001)
+    @cache.write(6, "ffffffffff") && sleep(0.001)
+    @cache.write(7, "gggggggggg") && sleep(0.001)
+    @cache.write(8, "hhhhhhhhhh") && sleep(0.001)
+    @cache.write(9, "iiiiiiiiii") && sleep(0.001)
+    long_key = '*' * 2 * @record_size
+    @cache.write(long_key, "llllllllll")
+    assert @cache.exist?(long_key)
+    assert @cache.exist?(9)
+    assert @cache.exist?(8)
+    assert @cache.exist?(7)
+    assert @cache.exist?(6)
+    assert !@cache.exist?(5), "no entry"
+    assert !@cache.exist?(4), "no entry"
+    assert !@cache.exist?(3), "no entry"
+    assert !@cache.exist?(2), "no entry"
+    assert !@cache.exist?(1), "no entry"
   end
 
   def test_pruning_is_capped_at_a_max_time
@@ -692,54 +861,83 @@ class MemoryStoreTest < ActiveSupport::TestCase
     assert @cache.exist?(2)
     assert !@cache.exist?(1)
   end
+
+  def test_write_with_unless_exist
+    assert_equal true, @cache.write(1, "aaaaaaaaaa")
+    assert_equal false, @cache.write(1, "aaaaaaaaaa", :unless_exist => true)
+    @cache.write(1, nil)
+    assert_equal false, @cache.write(1, "aaaaaaaaaa", :unless_exist => true)
+  end
 end
 
-uses_memcached 'memcached backed store' do
-  class MemCacheStoreTest < ActiveSupport::TestCase
-    def setup
-      @cache = ActiveSupport::Cache.lookup_store(:mem_cache_store, :expires_in => 60)
-      @peek = ActiveSupport::Cache.lookup_store(:mem_cache_store)
-      @data = @cache.instance_variable_get(:@data)
-      @cache.clear
-      @cache.silence!
-    end
+class MemCacheStoreTest < ActiveSupport::TestCase
+  require 'dalli'
 
-    include CacheStoreBehavior
-    include LocalCacheBehavior
-    include CacheIncrementDecrementBehavior
-    include EncodedKeyCacheBehavior
+  begin
+    ss = Dalli::Client.new('localhost:11211').stats
+    raise Dalli::DalliError unless ss['localhost:11211']
 
-    def test_raw_values
-      cache = ActiveSupport::Cache.lookup_store(:mem_cache_store, :raw => true)
-      cache.clear
+    MEMCACHE_UP = true
+  rescue Dalli::DalliError
+    $stderr.puts "Skipping memcached tests. Start memcached and try again."
+    MEMCACHE_UP = false
+  end
+
+  def setup
+    skip "memcache server is not up" unless MEMCACHE_UP
+
+    @cache = ActiveSupport::Cache.lookup_store(:mem_cache_store, :expires_in => 60)
+    @peek = ActiveSupport::Cache.lookup_store(:mem_cache_store)
+    @data = @cache.instance_variable_get(:@data)
+    @cache.clear
+    @cache.silence!
+    @cache.logger = ActiveSupport::Logger.new("/dev/null")
+  end
+
+  include CacheStoreBehavior
+  include LocalCacheBehavior
+  include CacheIncrementDecrementBehavior
+  include EncodedKeyCacheBehavior
+  include AutoloadingCacheBehavior
+
+  def test_raw_values
+    cache = ActiveSupport::Cache.lookup_store(:mem_cache_store, :raw => true)
+    cache.clear
+    cache.write("foo", 2)
+    assert_equal "2", cache.read("foo")
+  end
+
+  def test_raw_values_with_marshal
+    cache = ActiveSupport::Cache.lookup_store(:mem_cache_store, :raw => true)
+    cache.clear
+    cache.write("foo", Marshal.dump([]))
+    assert_equal [], cache.read("foo")
+  end
+
+  def test_local_cache_raw_values
+    cache = ActiveSupport::Cache.lookup_store(:mem_cache_store, :raw => true)
+    cache.clear
+    cache.with_local_cache do
       cache.write("foo", 2)
       assert_equal "2", cache.read("foo")
     end
+  end
 
-    def test_raw_values_with_marshal
-      cache = ActiveSupport::Cache.lookup_store(:mem_cache_store, :raw => true)
-      cache.clear
+  def test_local_cache_raw_values_with_marshal
+    cache = ActiveSupport::Cache.lookup_store(:mem_cache_store, :raw => true)
+    cache.clear
+    cache.with_local_cache do
       cache.write("foo", Marshal.dump([]))
       assert_equal [], cache.read("foo")
     end
+  end
 
-    def test_local_cache_raw_values
-      cache = ActiveSupport::Cache.lookup_store(:mem_cache_store, :raw => true)
-      cache.clear
-      cache.with_local_cache do
-        cache.write("foo", 2)
-        assert_equal "2", cache.read("foo")
-      end
-    end
-
-    def test_local_cache_raw_values_with_marshal
-      cache = ActiveSupport::Cache.lookup_store(:mem_cache_store, :raw => true)
-      cache.clear
-      cache.with_local_cache do
-        cache.write("foo", Marshal.dump([]))
-        assert_equal [], cache.read("foo")
-      end
-    end
+  def test_read_should_return_a_different_object_id_each_time_it_is_called
+    @cache.write('foo', 'bar')
+    assert_not_equal @cache.read('foo').object_id, @cache.read('foo').object_id
+    value = @cache.read('foo')
+    value << 'bingo'
+    assert_not_equal value, @cache.read('foo')
   end
 end
 
@@ -806,30 +1004,21 @@ class CacheStoreLoggerTest < ActiveSupport::TestCase
     @cache = ActiveSupport::Cache.lookup_store(:memory_store)
 
     @buffer = StringIO.new
-    @cache.logger = Logger.new(@buffer)
+    @cache.logger = ActiveSupport::Logger.new(@buffer)
   end
 
   def test_logging
     @cache.fetch('foo') { 'bar' }
-    assert_present @buffer.string
+    assert @buffer.string.present?
   end
 
   def test_mute_logging
     @cache.mute { @cache.fetch('foo') { 'bar' } }
-    assert_blank @buffer.string
+    assert @buffer.string.blank?
   end
 end
 
 class CacheEntryTest < ActiveSupport::TestCase
-  def test_create_raw_entry
-    time = Time.now
-    entry = ActiveSupport::Cache::Entry.create("raw", time, :compress => false, :expires_in => 300)
-    assert_equal "raw", entry.raw_value
-    assert_equal time.to_f, entry.created_at
-    assert !entry.compressed?
-    assert_equal 300, entry.expires_in
-  end
-
   def test_expired
     entry = ActiveSupport::Cache::Entry.new("value")
     assert !entry.expired?, 'entry not expired'
@@ -841,16 +1030,42 @@ class CacheEntryTest < ActiveSupport::TestCase
   end
 
   def test_compress_values
-    entry = ActiveSupport::Cache::Entry.new("value", :compress => true, :compress_threshold => 1)
-    assert_equal "value", entry.value
-    assert entry.compressed?
-    assert_equal "value", Marshal.load(Zlib::Inflate.inflate(entry.raw_value))
+    value = "value" * 100
+    entry = ActiveSupport::Cache::Entry.new(value, :compress => true, :compress_threshold => 1)
+    assert_equal value, entry.value
+    assert(value.bytesize > entry.size, "value is compressed")
   end
 
   def test_non_compress_values
-    entry = ActiveSupport::Cache::Entry.new("value")
-    assert_equal "value", entry.value
-    assert_equal "value", Marshal.load(entry.raw_value)
-    assert !entry.compressed?
+    value = "value" * 100
+    entry = ActiveSupport::Cache::Entry.new(value)
+    assert_equal value, entry.value
+    assert_equal value.bytesize, entry.size
+  end
+
+  def test_restoring_version_4beta1_entries
+    version_4beta1_entry = ActiveSupport::Cache::Entry.allocate
+    version_4beta1_entry.instance_variable_set(:@v, "hello")
+    version_4beta1_entry.instance_variable_set(:@x, Time.now.to_i + 60)
+    entry = Marshal.load(Marshal.dump(version_4beta1_entry))
+    assert_equal "hello", entry.value
+    assert_equal false, entry.expired?
+  end
+
+  def test_restoring_compressed_version_4beta1_entries
+    version_4beta1_entry = ActiveSupport::Cache::Entry.allocate
+    version_4beta1_entry.instance_variable_set(:@v, Zlib::Deflate.deflate(Marshal.dump("hello")))
+    version_4beta1_entry.instance_variable_set(:@c, true)
+    entry = Marshal.load(Marshal.dump(version_4beta1_entry))
+    assert_equal "hello", entry.value
+  end
+
+  def test_restoring_expired_version_4beta1_entries
+    version_4beta1_entry = ActiveSupport::Cache::Entry.allocate
+    version_4beta1_entry.instance_variable_set(:@v, "hello")
+    version_4beta1_entry.instance_variable_set(:@x, Time.now.to_i - 1)
+    entry = Marshal.load(Marshal.dump(version_4beta1_entry))
+    assert_equal true, entry.expired?
+    assert_equal "hello", entry.value
   end
 end
